
/************************************************************************/
/*                                                                      */
/*	Load and Print Bios Parameter Block Data - LOADBPB.C		*/
/*                                                                      */
/*	Version 1.1, July  1987 					*/
/*                                                                      */
/*	Compiled under Lattice C version 3.20	(Large model -mls)	*/
/*                                                                      */
/*	Author: Lawrence E. Himes					*/
/*                                                                      */
/*	Rev 1.1 Corrects Bug in Calculation of FAT entry count, etc	*/
/*                                                                      */
/************************************************************************/

#include "bpb.h"
#include <stdio.h>
#include "dglobal.h"

/****************************************************************/
/*                                                              */
/*      loadbpb  () - Loads BPB info from boot sector of drive  */
/*                                                              */
/****************************************************************/

loadbpb (drv)
  int drv;
{
  int r, t;
  struct bpb *bp;
  char sbuf [512];              /* Sector buffer  */
  void printbpb();

        r = absdio (0, drive = drv, sbuf, 0, 1);      /* Load boot block */
        if (!r)  {
                bp = (struct bpb *)sbuf;
                secsize = bp->s_bytesper;       /* Load global params */
                clsecs  = bp->c_sectors;
                clsize  = secsize * clsecs;
                resrvs  = bp->resrvsec;
                nfats   = bp->Nfats;
                NRoots  = bp->Nrootdir;
                Rootsize = NRoots * 32;
                Rootsecs = Rootsize / secsize;
		Fatsecs = bp->Fat_secsper;
		t = Fatsecs * nfats;
		Fatents = (bp->Tsectors - t - Rootsecs - resrvs) / clsecs;
		Frootsec = resrvs + t;
		Fatsize  = Fatsecs * secsize;
		Algo =	(Fatents > 0x0FF0);
                Ffilesec = Frootsec + Rootsecs;
                printbpb (bp);
        }
        return (r);
}

void
printbpb (bp)
  struct bpb *bp;
{
  char n [10];

        movmem ( bp->oemname, n, 8);
        n [8] = 0;
        cprintf ("\nOEM Name        : %s", n);
        cprintf ("\nBytes per sector: %d", bp->s_bytesper);
        cprintf ("\nSectors per clus: %d", bp->c_sectors);
        cprintf ("\nReserved Sectors: %d", bp->resrvsec);
        cprintf ("\nNumber of FATs  : %d", bp->Nfats);
        cprintf ("\nRoot Dir. Ents. : %d", bp->Nrootdir);
        cprintf ("\nTotal Sectors   : %u", bp->Tsectors);
        cprintf ("\nMedia Descriptor: %X", bp->Media);
        cprintf ("\nSectors per FAT : %d", bp->Fat_secsper);
        cprintf ("\nSectors per Trk.: %d", bp->Trk_secsper);
        cprintf ("\nNumber of Heads : %d", bp->Nheads );
        cprintf ("\nHidden Sectors  : %d", bp->Nhiden );
        cprintf ("\nRoot Dir Sectors: %d", Rootsecs   );
        cprintf ("\nFAT Entries     : %u\n", Fatents    );
}

